<?php

namespace Drupal\Tests\aws_cloud\Functional\Ec2;

use Drupal\Component\Utility\Random;

use Drupal\Tests\aws_cloud\Functional\Utils;

// Updated by yas 2016/06/23
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Updated by yas 2016/05/28
// Updated by yas 2016/05/25
// Updated by yas 2016/05/24
// Created by yas 2016/05/23.

/**
 * Tests AWS Cloud Snapshot.
 *
 * @group AWS Cloud
 */
class SnapshotTest extends AwsCloudTestCase {

  const AWS_CLOUD_SNAPSHOT_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list aws cloud snapshot',
      'add aws cloud snapshot',
      'view aws cloud snapshot',
      'edit aws cloud snapshot',
      'delete aws cloud snapshot',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'snapshot_id' => 'snap-' . $this->random->name(17, TRUE),
      'cidr_block' => Utils::getRandomCidr(),
      'group_id' => 'sg-' . $this->random->name(8, TRUE),
      'start_time' => date('c'),
    ];
  }

  /**
   * Tests CRUD for Snapshot information.
   */
  public function testSnapshot() {
    $cloud_context = $this->cloud_context;

    // List Snapshot for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
    $this->assertResponse(200, t('List | HTTP 200: Snapshot'));
    $this->assertNoText(t('Notice'), t('List | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('List | Make sure w/o Warnings'));

    // Add a new Snapshot.
    $add = $this->createSnapshotTestData();
    for ($i = 0; $i < self::AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $num = $i + 1;

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/snapshot/add",
                            $add[$i],
                            t('Save'));

      $this->assertResponse(200, t('HTTP 200: Add | A New AWS Cloud Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Snapshot "@name', [
                        '@name' => $add[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('The AWS Cloud Snapshot "@name" has been created.', [
                        '@name' => $add[$i]['name'],
                      ]));
      $this->assertText($add[$i]['name'],
                     t('key_pair: @name ', [
                       '@name' => $add[$i]['name'],
                     ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
      $this->assertResponse(200, t('HTTP 200: List | Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | List | Make sure w/o Warnings'));
    }

    // Edit an Snapshot information.
    $edit = $this->createSnapshotTestData();
    for ($i = 0; $i < self::AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      unset($edit[$i]['volume_id']);
      unset($edit[$i]['description']);

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/snapshot/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->assertResponse(200, t('Edit | HTTP 200: A New AWS Cloud Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Snapshot "@name" has been saved.', [
        '@name' => $edit[$i]['name'],
      ]),
                        t('Confirm Message') . ': '
                      . t('The AWS Cloud Snapshot "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));

      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($edit[$i]['name'],
                        t('Edit | List | Make sure w/ Listing: @name', [
                          '@name' => $edit[$i]['name'],
                        ]));
      }
    }

    // Delete Snapshot.
    for ($i = 0; $i < self::AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot/$num/delete");
      $this->assertResponse(200, t('Delete | HTTP 200: Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/snapshot/$num/delete",
                            [],
                            t('Delete'));

      $this->assertResponse(200, t('Delete | HTTP 200: The Cloud Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->assertText($edit[$i]['name'], t('Name: @name ', ['@name' => $edit[$i]['name']]));
      $this->assertText(t('The AWS Cloud Snapshot "@name" has been deleted.', ['@name' => $edit[$i]['name']]),
        t('Confirm Message') . ': ' . t('Delete | The AWS Cloud Snapshot "@name" has been deleted.', ['@name' => $edit[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
      $this->assertResponse(200, t('Delete | HTTP 200: Snapshot', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | List | Make sure w/o Warnings'));
    }
  }

  /**
   * Test updating snapshots.
   */
  public function testUpdateSnapshot() {
    $this->_testUpdateSnapshotRepeat($max_count = self::AWS_CLOUD_SNAPSHOT_REPEAT_COUNT);
  }

  private function _testUpdateSnapshotRepeat($max_count) {
    $cloud_context = $this->cloud_context;

    for ($i = 0; $i < $max_count; $i++) {
      $num = $i + 1;
      $test_cases = $this->createUpdateSnapshotTestCases();
      $this->updateDescribeSnapshotsInMockData($test_cases);
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot/update");
      $this->assertResponse(200, t('Update | HTTP 200: Snapshot', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Update | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Update | List | Make sure w/o Warnings'));

      foreach ($test_cases as $test_case) {
        $this->assertLink(
          isset($test_case['name'])
          ? $test_case['name']
          : $test_case['id']
        );
      }
    }
  }

  private function createUpdateSnapshotTestCases() {
    $test_cases = [];
    $random = $this->random;

    // Only id.
    $test_cases[] = ['id' => 'snap-' . $random->name(32, TRUE)];

    // Name and id.
    $test_cases[] = [
      'name' => 'Snapshot Name ' . $random->name(32, TRUE),
      'id' => 'snap-' . $random->name(32, TRUE),
    ];

    return $test_cases;
  }

  private function updateDescribeSnapshotsInMockData($test_cases) {
    $random = $this->random;

    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeSnapshots'] = ['Snapshots' => []];
    foreach ($test_cases as $test_case) {
      $snapshot = [
        'SnapshotId' => $test_case['id'],
        'VolumeSize' => 10,
        'Description' => $random->string(32, TRUE),
        'State' => 'completed',
        'VolumeId' => 'vol-' . $random->name(32, TRUE),
        'Progress' => '100%',
        'Encrypted' => TRUE,
        'KmsKeyId' => 'arn:aws:kms:us-east-1:123456789012:key/6876fb1b-example',
        'OwnerId' => strval(rand(100000000000, 999999999999)),
        'OwnerAlias' => 'amazon',
        'StateMessage' => $random->string(32, TRUE),
        'StartTime' => date('c'),
      ];

      if (isset($test_case['name'])) {
        $snapshot['Tags'] = [
          ['Key' => 'Name', 'Value' => $test_case['name']],
        ];
      }

      $mock_data['DescribeSnapshots']['Snapshots'][] = $snapshot;
    }

    $this->updateMockDataToConfig($mock_data);
  }

  private function createSnapshotTestData() {
    $data = [];

    for ($i = 0; $i < self::AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Input Fields.
      $data[$i] = [
        'name'        => "Name #$num - " . $this->random->name(32, TRUE),
        'volume_id'   => "vol-" . $this->random->name(8, TRUE),
        'description' => "Description #$num - " . $this->random->name(64, TRUE),
      ];
    }
    return $data;
  }

}
